--- old/usr/src/common/zfs/zfeature_common.c	Tue Oct  1 14:04:10 2013
+++ new/usr/src/common/zfs/zfeature_common.c	Tue Oct  1 14:04:10 2013
@@ -92,32 +92,22 @@
 	if (zfeature_checks_disable)
 		return (B_TRUE);
 
-	return (0 == zfeature_lookup_guid(guid, NULL));
-}
-
-int
-zfeature_lookup_guid(const char *guid, zfeature_info_t **res)
-{
-	for (int i = 0; i < SPA_FEATURES; i++) {
+	for (spa_feature_t i = 0; i < SPA_FEATURES; i++) {
 		zfeature_info_t *feature = &spa_feature_table[i];
-		if (strcmp(guid, feature->fi_guid) == 0) {
-			if (res != NULL)
-				*res = feature;
-			return (0);
-		}
+		if (strcmp(guid, feature->fi_guid) == 0)
+			return (B_TRUE);
 	}
-
-	return (ENOENT);
+	return (B_FALSE);
 }
 
 int
-zfeature_lookup_name(const char *name, zfeature_info_t **res)
+zfeature_lookup_name(const char *name, spa_feature_t *res)
 {
-	for (int i = 0; i < SPA_FEATURES; i++) {
+	for (spa_feature_t i = 0; i < SPA_FEATURES; i++) {
 		zfeature_info_t *feature = &spa_feature_table[i];
 		if (strcmp(name, feature->fi_uname) == 0) {
 			if (res != NULL)
-				*res = feature;
+				*res = i;
 			return (0);
 		}
 	}
@@ -126,11 +116,12 @@
 }
 
 static void
-zfeature_register(int fid, const char *guid, const char *name, const char *desc,
-    boolean_t readonly, boolean_t mos, zfeature_info_t **deps)
+zfeature_register(spa_feature_t fid, const char *guid, const char *name,
+    const char *desc, boolean_t readonly, boolean_t mos,
+    const spa_feature_t *deps)
 {
 	zfeature_info_t *feature = &spa_feature_table[fid];
-	static zfeature_info_t *nodeps[] = { NULL };
+	static spa_feature_t nodeps[] = { SPA_FEATURE_NONE };
 
 	ASSERT(name != NULL);
 	ASSERT(desc != NULL);
@@ -141,6 +132,7 @@
 	if (deps == NULL)
 		deps = nodeps;
 
+	feature->fi_feature = fid;
 	feature->fi_guid = guid;
 	feature->fi_uname = name;
 	feature->fi_desc = desc;
@@ -167,4 +159,8 @@
 	zfeature_register(SPA_FEATURE_SPACEMAP_HISTOGRAM,
 	    "com.delphix:spacemap_histogram", "spacemap_histogram",
 	    "Spacemaps maintain space histograms.", B_TRUE, B_FALSE, NULL);
+	zfeature_register(SPA_FEATURE_EXTENSIBLE_DATASET,
+	    "com.delphix:extensible_dataset", "extensible_dataset",
+	    "Enhanced dataset functionality, used by other features.",
+	    B_FALSE, B_FALSE, NULL);
 }
